@@ -159,7 +159,7 @@ public:
   void updateVariablesTreeItem(VariablesTreeItem *pVariablesTreeItem);
   QModelIndex variablesTreeItemIndex(const VariablesTreeItem *pVariablesTreeItem) const;
   bool insertVariablesItems(QString fileName, QString filePath, QStringList variablesList, SimulationOptions simulationOptions);
-  void parseInitXml(QXmlStreamReader &xmlReader, QStringList *variablesList);
+  void parseInitXml(QXmlStreamReader &xmlReader, SimulationOptions simulationOptions, QStringList *variablesList);
   bool removeVariableTreeItem(QString variable);
   void unCheckVariables(VariablesTreeItem *pVariablesTreeItem);
   void plotAllVariables(VariablesTreeItem *pVariablesTreeItem, OMPlot::PlotWindow *pPlotWindow);
@@ -205,6 +205,7 @@ typedef struct {
   QString variableName;
   QString unit;
   QString displayUnit;
+  bool isString;
 } PlotParametricVariable;
 
 typedef struct {
@@ -233,7 +234,7 @@ public:
   void reSimulate(bool showSetup);
   void interactiveReSimulation(QString modelName);
   void updateInitXmlFile(SimulationOptions simulationOptions);
-  void initializeVisualization(SimulationOptions simulationOptions);
+  void initializeVisualization();
   double readVariableValue(QString variable, double time);
   void closeResultFile();
 private:
@@ -261,7 +262,7 @@ private:
   csv_data *mpCSVData;
   QFile mPlotFileReader;
   void selectInteractivePlotWindow(VariablesTreeItem *pVariablesTreeItem);
-  void openResultFile();
+  void openResultFile(double &startTime, double &stopTime);
   void updateVisualization();
   void checkVariable(const QModelIndex &index, bool checkState);
   void unCheckVariableAndErrorMessage(const QModelIndex &index, const QString &errorMessage);
