@@ -48,11 +48,13 @@ public:
   Legend(Plot *pParent);
   bool eventFilter(QObject *object, QEvent *event);
 public slots:
+  void toggleSign(bool checked);
   void showSetupDialog();
   void legendMenu(const QPoint&);
 private:
   Plot *mpPlot;
   PlotCurve *mpPlotCurve;
+  QAction *mpToggleSignAction;
   QAction *mpSetupAction;
 protected:
   virtual QWidget *createWidget(const QwtLegendData &data) const;
