@@ -60,7 +60,7 @@ oms::ComponentFMUCS::~ComponentFMUCS()
   fmi_import_free_context(context);
 }
 
-oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::System* parentSystem, const std::string& fmuPath)
+oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::System* parentSystem, const std::string& fmuPath, std::string replaceComponent)
 {
   if (!cref.isValidIdent())
   {
@@ -74,13 +74,50 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
     return NULL;
   }
 
+  // replaceComponent string will be used to avoid name conflicts when replacing a fmu with oms_replaceSubModel(), the default is ""
+
   filesystem::path temp_root(parentSystem->getModel().getTempDirectory());
   filesystem::path temp_temp = temp_root / "temp";
-  filesystem::path relFMUPath = parentSystem->copyResources() ? (filesystem::path("resources") / (parentSystem->getUniqueID() + "_" + std::string(cref) + ".fmu")) : filesystem::path(fmuPath);
+  filesystem::path relFMUPath = parentSystem->copyResources() ? (filesystem::path("resources") / (parentSystem->getUniqueID() + "_" + replaceComponent + std::string(cref) + ".fmu")) : filesystem::path(fmuPath);
   filesystem::path absFMUPath = temp_root / relFMUPath;
 
   ComponentFMUCS* component = new ComponentFMUCS(cref, parentSystem, relFMUPath.generic_string());
 
+  /* parse the modeldescription.xml at top level to get the GUID to check whether instance already exist
+   * so we don't need to unpack the fmu, and also parse start values before instantiating fmu's
+  */
+  std::string guid_ = "";
+  filesystem::path modelDescriptionPath;
+  /*
+  * check for modeldescription path from file system or temp directory
+  * because when importingSnapshot the path will be resources/0001_tank1.fmu
+  */
+  if (parentSystem->copyResources())
+    modelDescriptionPath = fmuPath;
+  else
+    modelDescriptionPath = parentSystem->getModel().getTempDirectory() / filesystem::path(fmuPath);
+
+  component->values.parseModelDescription(modelDescriptionPath, guid_);
+
+  /*
+   * check if instance of an fmu already exist by using guid of the fmu
+   * if instance exist use the existing instance path
+   * eg: tank1 => resources /0001_tank1.fmu
+   *     tank2 => resources /0001_tank1.fmu
+  */
+  auto it = parentSystem->fmuGuid.find(guid_);
+  if (it == parentSystem->fmuGuid.end())
+    parentSystem->fmuGuid[guid_] = relFMUPath;
+  else
+  {
+    // instance exists and update the FMU path to already existing instance
+    relFMUPath = parentSystem->fmuGuid[guid_];
+    absFMUPath = temp_root / relFMUPath;
+    component->setPath(relFMUPath.generic_string());
+  }
+
+  //std::cout <<"\nFmuInstances: " << cref.c_str() << "=" << guid_ << "=" << relFMUPath.generic_string();
+
   component->callbacks.malloc = malloc;
   component->callbacks.calloc = calloc;
   component->callbacks.realloc = realloc;
@@ -91,22 +128,33 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
 
   // Copy the resource to the temp directory of the model? We don't want have
   // to copy resources if importing an SSP file or snapshot.
-  if (parentSystem->copyResources())
+  if (parentSystem->copyResources() && !filesystem::exists(absFMUPath))
     oms_copy_file(filesystem::path(fmuPath), absFMUPath);
 
   // set temp directory
   filesystem::path tempDir = temp_temp / relFMUPath.stem();
   component->setTempDir(tempDir.string());
-  if (!filesystem::is_directory(tempDir) && !filesystem::create_directory(tempDir))
+
+  bool dirExist = true;
+  if (!filesystem::is_directory(tempDir))
   {
-    logError("Creating temp directory for component \"" + std::string(cref) + "\" failed");
-    return NULL;
+    dirExist = false;
+    if (!filesystem::create_directory(tempDir))
+    {
+      logError("Creating temp directory for component \"" + std::string(cref) + "\" failed");
+      return NULL;
+    }
   }
 
   component->context = fmi_import_allocate_context(&component->callbacks);
 
   // check version of FMU
-  fmi_version_enu_t version = fmi_import_get_fmi_version(component->context, absFMUPath.string().c_str(), component->getTempDir().c_str());
+  fmi_version_enu_t version;
+  if (dirExist)
+    version = fmi_import_get_fmi_version_unzipped(component->context, tempDir.generic_string().c_str());
+  else
+    version = fmi_import_get_fmi_version(component->context, absFMUPath.string().c_str(), component->getTempDir().c_str());
+
   if (fmi_version_2_0_enu != version)
   {
     logError("Unsupported FMI version: " + std::string(fmi_version_to_string(version)));
@@ -235,13 +283,21 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
     return NULL;
   }
 
-  // parse modelDescription.xml to get start values before instantiating fmu's
-  component->values.parseModelDescription(tempDir);
-
+  // set units to connector
+  for (auto &connector : component->connectors)
+  {
+    if (connector)
+    {
+      oms::ComRef connectorCref = connector->getName();
+      std::string unitName = component->values.getUnitFromModeldescription(connectorCref);
+      if (!unitName.empty())
+        connector->connectorUnits[unitName] = component->values.modeldescriptionUnitDefinitions[unitName];
+    }
+  }
   return component;
 }
 
-oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, oms::System* parentSystem, const std::string& sspVersion, const Snapshot& snapshot)
+oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, oms::System* parentSystem, const std::string& sspVersion, const Snapshot& snapshot, std::string variantName)
 {
   ComRef cref = ComRef(node.attribute("name").as_string());
   std::string type = node.attribute("type").as_string();
@@ -266,10 +322,20 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, om
     std::string name = it->name();
     if(name == oms::ssp::Draft20180219::ssd::connectors)
     {
+      // get the ssdNode to parse UnitDefinitions in "SystemStructure.ssd"
+      pugi::xml_node ssdNode = snapshot.getResourceNode(variantName);
+      component->values.importUnitDefinitions(ssdNode);
       // import connectors
       for(pugi::xml_node_iterator itConnectors = (*it).begin(); itConnectors != (*it).end(); ++itConnectors)
       {
         component->connectors.push_back(oms::Connector::NewConnector(*itConnectors, sspVersion, component->getFullCref()));
+        // set units to connector
+        if ((*itConnectors).child(oms::ssp::Version1_0::ssc::real_type))
+        {
+          std::string unitName = (*itConnectors).child(oms::ssp::Version1_0::ssc::real_type).attribute("unit").as_string();
+          if (!unitName.empty())
+            component->connectors.back()->connectorUnits[unitName] = component->values.modeldescriptionUnitDefinitions[unitName];
+        }
       }
     }
     else if(name == oms::ssp::Draft20180219::ssd::element_geometry)
@@ -283,7 +349,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, om
       // set parameter bindings associated with the component
       Values resources;
       std::string tempdir = parentSystem->getModel().getTempDirectory();
-      resources.importFromSnapshot(*it, sspVersion, snapshot);
+      resources.importFromSnapshot(*it, sspVersion, snapshot, variantName);
       component->values.parameterResources.push_back(resources);
     }
     else
@@ -300,7 +366,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, om
   return component;
 }
 
-oms_status_enu_t oms::ComponentFMUCS::exportToSSD(pugi::xml_node& node, Snapshot& snapshot) const
+oms_status_enu_t oms::ComponentFMUCS::exportToSSD(pugi::xml_node& node, Snapshot& snapshot, std::string variantName) const
 {
 #if !defined(NO_TLM)
   if (tlmbusconnectors[0])
@@ -328,13 +394,30 @@ oms_status_enu_t oms::ComponentFMUCS::exportToSSD(pugi::xml_node& node, Snapshot
         return oms_status_error;
 
   // export ParameterBindings at component level
-  values.exportParameterBindings(node, snapshot);
+  values.exportParameterBindings(node, snapshot, variantName);
 
   return oms_status_ok;
 }
 
 void oms::ComponentFMUCS::getFilteredUnitDefinitionsToSSD(std::map<std::string, std::map<std::string, std::string>>& unitDefinitions)
 {
+  // get units from connectors
+  for (const auto &connector : connectors)
+  {
+    if (connector)
+    {
+      if (!connector->connectorUnits.empty())
+      {
+        for (auto &con : connector->connectorUnits)
+        {
+          auto unitvalue = unitDefinitions.find(con.first);
+          if (unitvalue == unitDefinitions.end())
+            unitDefinitions[con.first] = con.second;
+        }
+      }
+    }
+  }
+
   return values.getFilteredUnitDefinitionsToSSD(unitDefinitions);
 }
 
@@ -736,6 +819,8 @@ oms_status_enu_t oms::ComponentFMUCS::newResources(const std::string& ssvFilenam
       resources.ssmFile = "resources/" + ssmFilename;
     // copy modeldescriptionVariableUnits to ssv resources which will be used to export units
     resources.modelDescriptionVariableUnits = values.modelDescriptionVariableUnits;
+    // copy modeldescriptionVariableUnitDefinitions to ssv resources which will be used to export unit definitions
+    resources.modeldescriptionUnitDefinitions = values.modeldescriptionUnitDefinitions;
     values.parameterResources[0].allresources["resources/" + ssvFilename] = resources;
   }
 
@@ -1435,6 +1520,7 @@ oms_status_enu_t oms::ComponentFMUCS::setBoolean(const ComRef& cref, bool value)
 
   if (oms_modelState_virgin == getModel().getModelState())
   {
+    values.updateModelDescriptionBooleanStartValue(cref, value);
     // check for local resources available
     if (values.hasResources())
     {
@@ -1485,6 +1571,8 @@ oms_status_enu_t oms::ComponentFMUCS::setInteger(const ComRef& cref, int value)
 
   if (oms_modelState_virgin == getModel().getModelState())
   {
+    // update start values in top level Modeldescription.xml to be exported in ssv templates
+    values.updateModelDescriptionIntegerStartValue(cref, value);
     // check for local resources available
     if (values.hasResources())
     {
@@ -1538,19 +1626,24 @@ oms_status_enu_t oms::ComponentFMUCS::setReal(const ComRef& cref, double value)
 
   if (oms_modelState_virgin == getModel().getModelState())
   {
+    // update start values in top level Modeldescription.xml to be exported in ssv templates
+    values.updateModelDescriptionRealStartValue(cref, value);
     // check for local resources available
     if (values.hasResources())
     {
+      values.copyModelDescriptionUnitToResources(values);
       return values.setRealResources(cref, value, getFullCref(), false, oms_modelState_virgin);
     }
     // check for resources in root
     else if (getParentSystem() && getParentSystem()->getValues().hasResources())
     {
+      getParentSystem()->getValues().copyModelDescriptionUnitToResources(values);
       return getParentSystem()->getValues().setRealResources(getCref()+cref, value, getParentSystem()->getFullCref(), false, oms_modelState_virgin);
     }
     // check for resources in top level root
     else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
     {
+      getParentSystem()->getParentSystem()->getValues().copyModelDescriptionUnitToResources(values);
       return getParentSystem()->getParentSystem()->getValues().setRealResources(getCref()+cref, value, getParentSystem()->getParentSystem()->getFullCref(), false, oms_modelState_virgin);
     }
     else
@@ -1591,6 +1684,7 @@ oms_status_enu_t oms::ComponentFMUCS::setString(const ComRef& cref, const std::s
 
   if (oms_modelState_virgin == getModel().getModelState())
   {
+    values.updateModelDescriptionStringStartValue(cref, value);
     // check for local resources available
     if (values.hasResources())
     {
@@ -1625,6 +1719,22 @@ oms_status_enu_t oms::ComponentFMUCS::setString(const ComRef& cref, const std::s
 
 oms_status_enu_t oms::ComponentFMUCS::setUnit(const ComRef &cref, const std::string &value)
 {
+  // set units to connectors
+  for (auto &connector : connectors)
+  {
+    if (connector)
+    {
+      if (connector->getName() == cref)
+      {
+        connector->connectorUnits.clear();
+        connector->connectorUnits[value] = {};
+      }
+    }
+  }
+
+  // set unit in top level modeldescription.xml
+  values.updateModelDescriptionVariableUnit(cref, value);
+
   // check for local resources available
   if (values.hasResources())
   {
@@ -1673,6 +1783,43 @@ oms_status_enu_t oms::ComponentFMUCS::deleteStartValue(const ComRef& cref)
   return oms_status_error;
 }
 
+oms_status_enu_t oms::ComponentFMUCS::setValuesResources(std::vector<Values>& allValuesResources)
+{
+  this->values.parameterResources = allValuesResources;
+  return oms_status_ok;
+}
+
+std::vector<oms::Values> oms::ComponentFMUCS::getValuesResources()
+{
+  return this->values.parameterResources;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::updateOrDeleteStartValueInReplacedComponent(std::vector<std::string>& warningList)
+{
+  // check for local resources available
+  if (values.hasResources())
+  {
+    return values.updateOrDeleteStartValueInReplacedComponent(values, this->getCref(), warningList);
+  }
+  // check for resources in root
+  else if (getParentSystem() && getParentSystem()->getValues().hasResources())
+  {
+    return getParentSystem()->getValues().updateOrDeleteStartValueInReplacedComponent(values, this->getCref(), warningList);
+  }
+  // check for resources in top level root
+  else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
+  {
+    return getParentSystem()->getParentSystem()->getValues().updateOrDeleteStartValueInReplacedComponent(values, this->getCref(), warningList);
+  }
+  else
+  {
+    // inline parameter settings, no need to update the values
+    return oms_status_ok;
+  }
+
+  return oms_status_error;
+}
+
 oms_status_enu_t oms::ComponentFMUCS::registerSignalsForResultFile(ResultWriter& resultFile)
 {
   resultFileMapping.clear();
