@@ -106,10 +106,23 @@ oms_status_enu_t oms::SystemWC::exportToSSD_SimulationInformation(pugi::xml_node
 {
   /* ssd:SimulationInformation should be added as vendor specific annotations from Version 1.0 */
   pugi::xml_node node_simulation_information = node.append_child(oms::ssp::Version1_0::simulation_information);
+  pugi::xml_node node_solver;
+
+  if (solverMethod == oms_solver_wc_mav || solverMethod == oms_solver_wc_mav2)
+  {
+    node_solver = node_simulation_information.append_child(oms::ssp::Version1_0::VariableStepMaster);
+    node_solver.append_attribute("description") = getSolverName().c_str();
+    node_solver.append_attribute("initialStepSize") = std::to_string(initialStepSize).c_str();
+    node_solver.append_attribute("minimumStepSize") = std::to_string(minimumStepSize).c_str();
+    node_solver.append_attribute("maximumStepSize") = std::to_string(maximumStepSize).c_str();
+  }
+  else if (solverMethod == oms_solver_wc_ma)
+  {
+    node_solver = node_simulation_information.append_child(oms::ssp::Version1_0::FixedStepMaster);
+    node_solver.append_attribute("description") = getSolverName().c_str();
+    node_solver.append_attribute("stepSize") = std::to_string(maximumStepSize).c_str();
+  }
 
-  pugi::xml_node node_solver = node_simulation_information.append_child(oms::ssp::Version1_0::FixedStepMaster);
-  node_solver.append_attribute("description") = getSolverName().c_str();
-  node_solver.append_attribute("stepSize") = std::to_string(maximumStepSize).c_str();
   node_solver.append_attribute("absoluteTolerance") = std::to_string(absoluteTolerance).c_str();
   node_solver.append_attribute("relativeTolerance") = std::to_string(relativeTolerance).c_str();
 
@@ -120,24 +133,49 @@ oms_status_enu_t oms::SystemWC::importFromSSD_SimulationInformation(const pugi::
 {
   std::string solverName = "";
   const char* FixedStepMaster = "";
+  const char* VariableStepMaster = "";
 
   pugi::xml_node fixedStepMaster = node.child(oms::ssp::Version1_0::FixedStepMaster);
   if (fixedStepMaster)
   {
-    solverName = fixedStepMaster.attribute("description").as_string();
-    FixedStepMaster = oms::ssp::Version1_0::FixedStepMaster;
+    if (sspVersion == "1.0")
+    {
+      solverName = fixedStepMaster.attribute("description").as_string();
+      FixedStepMaster = oms::ssp::Version1_0::FixedStepMaster;
+    }
+    else
+    {
+      solverName = node.child("FixedStepMaster").attribute("description").as_string();
+      FixedStepMaster = "FixedStepMaster";
+    }
+    initialStepSize = minimumStepSize = maximumStepSize = node.child(FixedStepMaster).attribute("stepSize").as_double();
+    absoluteTolerance = node.child(FixedStepMaster).attribute("absoluteTolerance").as_double();
+    relativeTolerance = node.child(FixedStepMaster).attribute("relativeTolerance").as_double();
   }
-  else
+
+  pugi::xml_node variableStepMaster = node.child(oms::ssp::Version1_0::VariableStepMaster);
+  if (variableStepMaster)
   {
-    solverName = node.child("FixedStepMaster").attribute("description").as_string();
-    FixedStepMaster = "FixedStepMaster";
+    if (sspVersion == "1.0")
+    {
+      solverName = variableStepMaster.attribute("description").as_string();
+      VariableStepMaster = oms::ssp::Version1_0::VariableStepMaster;
+    }
+    else
+    {
+      solverName = node.child("VariableStepMaster").attribute("description").as_string();
+      VariableStepMaster = "VariableStepMaster";
+    }
+    minimumStepSize = node.child(VariableStepMaster).attribute("minimumStepSize").as_double();
+    maximumStepSize = node.child(VariableStepMaster).attribute("maximumStepSize").as_double();
+    initialStepSize = node.child(VariableStepMaster).attribute("initialStepSize").as_double();
+    absoluteTolerance = node.child(VariableStepMaster).attribute("absoluteTolerance").as_double();
+    relativeTolerance = node.child(VariableStepMaster).attribute("relativeTolerance").as_double();
   }
 
   if (oms_status_ok != setSolverMethod(solverName))
     return oms_status_error;
-  initialStepSize = minimumStepSize = maximumStepSize = node.child(FixedStepMaster).attribute("stepSize").as_double();
-  absoluteTolerance = node.child(FixedStepMaster).attribute("absoluteTolerance").as_double();
-  relativeTolerance = node.child(FixedStepMaster).attribute("relativeTolerance").as_double();
+
   return oms_status_ok;
 }
 
@@ -499,6 +537,8 @@ oms_status_enu_t oms::SystemWC::doStep()
 
     for (const auto& component : mav_canGetAndSetStateFMUcomponents)
       component.second->freeState();
+
+    return oms_status_ok;
   }
   else if (solverMethod == oms_solver_wc_ma)
   {
@@ -1006,6 +1046,14 @@ oms_status_enu_t oms::SystemWC::updateInputs(oms::DirectedGraph& graph)
       {
         double value = 0.0;
         if (oms_status_ok != getReal(graph.getNodes()[output].getName(), value)) return oms_status_error;
+        /* check for unit conversion and suppressUnitConversion and set the value multiplied by factor,
+         * by default, factor = 1.0, (e.g) mm to m will be (factor*value) => (10^-3 * value)
+        */
+        if (sortedConnections[i].suppressUnitConversion)
+          value = value;
+        else
+          value = sortedConnections[i].factor*value;
+
         if (oms_status_ok != setReal(graph.getNodes()[input].getName(), value)) return oms_status_error;
 
         // derivatives
